﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("MCVTripleMoveAttribute", "Specifies the tabu attributes for a change position move on an multi component vector.")]
  [StorableClass]
  public class MCVTripleMoveAttribute : MultiComponentVectorMoveAttribute {
    [Storable]
    public int Index { get; protected set; } 
    [Storable]
    public int ItemID { get; protected set; }  
    [Storable]
    public int TargetIndex { get; protected set; }
    [Storable]
    public bool Rotation { get; protected set; }
    [Storable]
    public int GroupBefore { get; protected set; }
    [Storable]
    public int GroupAfter { get; protected set; }

    [StorableConstructor]
    protected MCVTripleMoveAttribute(bool deserializing) : base(deserializing) { }
    protected MCVTripleMoveAttribute(MCVTripleMoveAttribute original, Cloner cloner)
      : base(original, cloner) {
        this.Index = original.Index;
        this.TargetIndex = original.TargetIndex;      
      this.ItemID = original.ItemID;
      this.Rotation = original.Rotation;
      this.GroupBefore = original.GroupBefore;
      this.GroupAfter = original.GroupAfter;
    }
    public MCVTripleMoveAttribute() : this(-1, -1, -1, false, -1, -1, -1) { }
    public MCVTripleMoveAttribute(int index, int targetIndex, int itemID, bool rotation, int groupBefore, int groupAfter, double moveQuality)
      : base(moveQuality) {
        Index = index;
        TargetIndex = targetIndex;
        ItemID = itemID;
        Rotation = rotation;
        GroupBefore = groupBefore;
        GroupAfter = groupAfter;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MCVTripleMoveAttribute(this, cloner);
    }
  }
}
